﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using userService;
using scheduleService;

public partial class _Default : System.Web.UI.Page
{
    UserService webService = new UserService();
    ScheduleService scheduleService = new ScheduleService();
    UserProfile user;
    List<Course> courses;

    protected void Page_Load(object sender, EventArgs e)
    {
        //chkboxOutsideStudyprogram.Checked = false;
        lblLoginMessage.Text = "";
        if (!Page.IsPostBack && user == null)
        {
            multiView.ActiveViewIndex = 0;
            courses = new List<Course>();
            ViewState["courses"] = courses;
        }
        if (courses == null)
        {
            courses = new List<Course>();
        }
        txtSearchCourses.Attributes.Add("infotext", "Type a course code or subject title...");
        txtSearchCourses.Attributes.Add("onfocus", "clearText(this)");
        txtSearchCourses.Attributes.Add("onblur", "resetText(this)");
    }
    protected void btnLogin_Click(object sender, EventArgs e)
    {
        if (txtUsername.Text.Length > 0 && txtPassword.Text.Length > 0)
        {
            string username = txtUsername.Text;
            // SHA1 hash
            System.Security.Cryptography.SHA1 sha = new System.Security.Cryptography.SHA1CryptoServiceProvider();
            Encoding enc = Encoding.Default;
            string pass_hash = BitConverter.ToString(sha.ComputeHash(enc.GetBytes(txtPassword.Text))).Replace("-", "");

            try
            {
                user = webService.Login(username, pass_hash);

                if (user.IsAuthenticated)
                {
                    // login successful
                    multiView.ActiveViewIndex = 1;
                }
                else
                {
                    lblLoginMessage.Text = "Incorrect username/password!";
                }
            }
            catch (Exception ex)
            {
                lblLoginMessage.Text = ex.Message;
            }
        }
    }

    protected void PopulateForm(object sender, EventArgs e)
    {
        userId.Value = user.Id.ToString();
        txtFullNameChange.Text = user.Name;
        txtUsernameChange.Text = user.Username;
        txtStudyProgramChange.Text = user.StudyProgram; /* --- */
        txtSearchCourses.Text = "Type a course code or subject title...";
        txtEmailChange.Text = user.Email;
        txtSemesterChange.Text = user.Semester.ToString();
        courses.AddRange(user.Courses);
        listCourseCodesChange.DataSource = courses;
        listCourseCodesChange.DataBind();
        ViewState["courses"] = courses; // To perserve values through postbacks
        lblStatusMessage.Text = "";
    }

    protected void addCourseToList(object sender, EventArgs e)
    {
        if (txtSearchCourses.Text.Length > 0 && txtSearchCourses.Text.IndexOf(' ') != -1)
        {
            if (scheduleService.CourseExists(txtSearchCourses.Text.Split(' ')[0]))
            {
                courses = ViewState["courses"] as List<Course>;
                Course course = new Course();
                course.Code = txtSearchCourses.Text.Split(' ')[0];
                course.Description = txtSearchCourses.Text.Substring(txtSearchCourses.Text.IndexOf(' ')).Trim();
                course.OutsideStudyProgram = chkboxOutsideStudyprogram.Checked;
                courses.Add(course);
                listCourseCodesChange.DataSource = courses;
                listCourseCodesChange.DataBind();
                txtSearchCourses.Text = "";
                // Update viewstate
                ViewState["courses"] = courses;
            }
            else
            {
                txtSearchCourses.Text = "";
                lblStatusMessage.Text = "Error: the course name is not valid. Use the dropdown list to select a course";
            }
        }
        else
        {
            txtSearchCourses.Text = "";
            lblStatusMessage.Text = "Error: the course name is not valid. Use the dropdown list to select a course";
        }
        chkboxOutsideStudyprogram.Checked = false;
    }

    protected void removeCourseFromList(object sender, EventArgs e)
    {
        courses.Clear();
        foreach (ListItem item in listCourseCodesChange.Items)
        {
            if (!item.Selected)
            {
                Course course = new Course();
                course.Code = item.Value.Split('#')[0];
                course.Description = item.Text;
                course.OutsideStudyProgram = bool.Parse(item.Value.Split('#')[1]);
                courses.Add(course);
            }
        }
        listCourseCodesChange.DataSource = courses;
        listCourseCodesChange.DataBind();
        ViewState["courses"] = courses;
    }

    protected void btnLogOut_Click(object sender, EventArgs e)
    {
        user = null;
        multiView.ActiveViewIndex = 0;
        Response.Redirect("Default.aspx");
    }

    protected void btnSave_Click(object sender, EventArgs e)
    {
        UserProfile updatedUser = new UserProfile();
        updatedUser.Id = Int32.Parse(userId.Value);
        updatedUser.Name = txtFullNameChange.Text;
        updatedUser.Email = txtEmailChange.Text;
        updatedUser.Username = txtUsernameChange.Text;
        updatedUser.StudyProgram = txtStudyProgramChange.Text;
        updatedUser.Courses = (ViewState["courses"] as List<Course>).ToArray();
        updatedUser.Semester = Int32.Parse(txtSemesterChange.Text);

        string passwordHash = "";
        if (txtPasswordChange.Text.Length > 0)
        {
            // SHA1 hash
            System.Security.Cryptography.SHA1 sha = new System.Security.Cryptography.SHA1CryptoServiceProvider();
            Encoding enc = Encoding.Default;
            passwordHash = BitConverter.ToString(sha.ComputeHash(enc.GetBytes(txtPasswordChange.Text))).Replace("-", "");
        }

        bool success = webService.UpdateUser(updatedUser, passwordHash);
        updatedUser = null;
        if (success)
        {
            lblStatusMessage.Text = "User profile successfully updated!";
        }
        else
        {
            lblStatusMessage.Text = "An error occured!";
        }
    }

    protected void listCourseCodesChange_DataBound(object sender, EventArgs e)
    {
        for (int i=0; i< listCourseCodesChange.Items.Count;i++)
        {
            listCourseCodesChange.Items[i].Value += ("#" + courses[i].OutsideStudyProgram);
        }
    }
}