﻿using System;
using System.IO;
using System.Net;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Data.SqlClient;

/// <summary>
/// Summary description for ScheduleServices
/// </summary>
[WebService(Namespace = "http://msis.myvnc.com/webservices/")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
// To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
[System.Web.Script.Services.ScriptService]
public class ScheduleService : System.Web.Services.WebService
{

    public ScheduleService()
    {

        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }

    /// <summary>
    /// This method returns a list of study programs. If a prefix is provided, only study programs matching 
    /// the prefix are included in the list. If the 'count' value is specified, the list is limited to
    /// 'count' number of elements.
    /// </summary>
    /// <param name="prefixText">Optional prefix (empty string for no prefix)</param>
    /// <param name="count">Optional number of elements to return (0 for all)</param>
    /// <returns></returns>
    [WebMethod]
    [System.Web.Script.Services.ScriptMethod]
    public string[] GetStudyProgramList(string prefixText, int count)
    {
        List<String> programList = new List<string>();

        SQLConn.Open();
        string queryString = "SELECT code FROM Program_Codes where code LIKE @prefix + '%' ORDER BY code ASC";

        using (SqlCommand cmd = SQLConn.ExecuteCommand(queryString))
        {
            cmd.Parameters.AddWithValue("prefix", prefixText);
            using (SqlDataReader dr = cmd.ExecuteReader())
            {
                while (dr.Read())
                {
                    programList.Add(dr["code"].ToString());
                }
            }
        }
        SQLConn.Close();
        return (count == 0) ? programList.ToArray() : programList.Take<string>(count).ToArray();
    }

    /// <summary>
    /// This method returns a list of available courses. If a prefix is provided, only courses matching 
    /// the prefix are included in the list. If the 'count' value is specified, the list is limited to
    /// 'count' number of elements.
    /// </summary>
    /// <param name="prefixText">Prefix (optional)</param>
    /// <param name="count">Number of elements to return (optional)</param>
    /// <returns></returns>
    [WebMethod]
    [System.Web.Script.Services.ScriptMethod]
    public string[] GetCourseList(string prefixText, int count)
    {
        List<String> courseList = new List<string>();

        SQLConn.Open();
        string queryString = "SELECT code, description "
                           + "FROM Courses WHERE code "
                           + "LIKE @prefix + '%' OR description LIKE '%' + @desc + '%' ORDER BY code ASC";

        using (SqlCommand cmd = SQLConn.ExecuteCommand(queryString))
        {
            cmd.Parameters.AddWithValue("prefix", prefixText);
            cmd.Parameters.AddWithValue("desc", prefixText);

            using (SqlDataReader dr = cmd.ExecuteReader())
            {
                while (dr.Read())
                {
                    courseList.Add(dr["code"].ToString().Trim() + " " + dr["description"].ToString().Trim());
                }
            }
        }
        SQLConn.Close();
        return courseList.ToArray();
    }

    [WebMethod]
    public bool CourseExists(string courseCode)
    {
        try
        {
            SQLConn.Open();
            SqlCommand cmd = SQLConn.ExecuteCommand("SELECT count(id) FROM Courses WHERE code = @courseCode");
            cmd.Parameters.AddWithValue("courseCode", courseCode);
            int result = (int)cmd.ExecuteScalar();
            if (result > 0)
            {
                return true;
            }
            return false;
        }
        catch (Exception e)
        {
            return false;
        }
    }

    /// <summary>
    /// This method returns all the lectures and exercise for courses a given user are attending.
    /// </summary>
    /// <param name="username">The username of the user</param>
    /// <returns>The list of lectures</returns>
    [WebMethod]
    public Appointment[] GetLectures(string username)
    {
        List<Appointment> lectures = new List<Appointment>();
        string[] courses = null; //
        string programCode = "";

        // Get courses and study program code for student

        if (!SQLConn.Open())
        {
            return null;
        }

        SqlCommand cmd = SQLConn.ExecuteCommand("SELECT courses, program_code FROM Users WHERE username='" + username + "'");
        if (cmd != null)
        {
            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.HasRows && reader.Read())
                {
                    if (reader["courses"] != "")
                    {
                        courses = (reader["courses"] as string).Split('|');
                        programCode = reader["program_code"].ToString().Trim();
                    }
                }
                reader.Close();
            }
        }
        SQLConn.Close();

        // Get lecture hours for each course
        if (courses != null)
        {
            for (int i = 0; i < courses.Length; i++)
            {
                // Strip off course description, and check if course should be excluded 
                // from study program code check.
                // courses[0]=course code, courses[1]=description, courses[2]=true|false (include in study program check)

                List<Appointment> a = GetLecturesForCourse(courses[i].Split('#')[0].Trim(),
                    bool.Parse(courses[i].Split('#')[2].Trim()) ? "" : programCode);
                if (a != null)
                {
                    lectures.AddRange(a);
                }
            }
        }

        return (lectures.Count > 0) ? lectures.ToArray() : null;
    }

    /// <summary>
    /// Return list of appointments (lectures) for a course, limited by program code.
    /// </summary>
    /// <param name="courseCode">The course code</param>
    /// <param name="programCode">The study program code. Leave empty to exclude study program check.</param>
    /// <returns>List of lectures matching course and program code</returns>
    private List<Appointment> GetLecturesForCourse(string courseCode, string programCode)
    {
        List<Appointment> appointments = new List<Appointment>();

        if (!SQLConn.Open())
        {
            return null;
        }

        string sqlQuery = "SELECT Timetable.*,description FROM Timetable LEFT JOIN Courses ON Timetable.course_code=Courses.code WHERE Timetable.course_code = @courseCode";
        using (SqlCommand cmd = SQLConn.ExecuteCommand(sqlQuery))
        {
            cmd.Parameters.AddWithValue("courseCode", courseCode);
            using (SqlDataReader dr = cmd.ExecuteReader())
            {
                while (dr.Read())
                {
                    string programCodes = dr["study_programs"].ToString();

                    if ((programCodes != "" && programCodes.Contains(programCode)) || programCodes == "" || programCode == "")
                    {
                        Appointment a = new Appointment();
                        a.Title = dr["course_code"].ToString() + " " + dr["description"].ToString();
                        if (dr["type"].ToString() == "Lecture")
                        {
                            a.AppointmentType = Appointment.Type.Lecture;
                        }
                        else if (dr["type"].ToString() == "Exercise")
                        {
                            a.AppointmentType = Appointment.Type.Exercise;
                        }
                        else
                        {
                            a.AppointmentType = Appointment.Type.Other;
                        }

                        // Add correct day
                        string day = dr["day"].ToString().ToLower();
                        if (day == "mandag") { a.Day = DayOfWeek.Monday; }
                        else if (day == "tirsdag") { a.Day = DayOfWeek.Tuesday; }
                        else if (day == "onsdag") { a.Day = DayOfWeek.Wednesday; }
                        else if (day == "torsdag") { a.Day = DayOfWeek.Thursday; }
                        else if (day == "fredag") { a.Day = DayOfWeek.Friday; }
                        else if (day == "lørdag") { a.Day = DayOfWeek.Saturday; }
                        else if (day == "søndag") { a.Day = DayOfWeek.Sunday; }

                        // Add correct starting and ending time 
                        a.StartHour = int.Parse(dr["start_time"].ToString().Split(':')[0]);
                        a.StartMinute = int.Parse(dr["start_time"].ToString().Split(':')[1]);
                        a.EndHour = int.Parse(dr["end_time"].ToString().Split(':')[0]);
                        a.EndMinute = int.Parse(dr["end_time"].ToString().Split(':')[1]);

                        // Add week numbers
                        List<int> weeks = new List<int>();
                        string[] weeks1 = dr["weeks"].ToString().Split(',');

                        for (int i = 0; i < weeks1.Length; i++)
                        {
                            if (weeks1[i].Contains('-'))
                            {
                                string[] tmp = weeks1[i].Split('-');
                                for (int j = Int32.Parse(tmp[0].Trim()); j <= Int32.Parse(tmp[1].Trim()); j++)
                                {
                                    weeks.Add(j);
                                }
                            }
                            else
                            {
                                weeks.Add(Int32.Parse(weeks1[i].Trim()));
                            }
                        }
                        a.Weeks = weeks.ToArray();                        

                        // Add weeks string
                        a.WeeksString = dr["weeks"].ToString();
                        
                        string weeksArray = "[";
                        foreach (int i in weeks)
                        {
                            weeksArray += i + "|";
                        }
                        
                        // Add room number
                        a.Room = dr["room"].ToString().Replace("#", ", ");

                        // Add teacher
                        a.Person = dr["teacher"].ToString();

                        // Check if an appointment with same day already exists.
                        // If so, only append room number to existing appointment.
                        // If not, add the new appointment

                        bool isDuplicate = false;
                        if (appointments.Count > 0)
                        {
                            for (int i = 0; i < appointments.Count; i++)
                            {
                                if (appointments[i].Title == a.Title && appointments[i].Day == a.Day && appointments[i].StartHour == a.StartHour)
                                {
                                    isDuplicate = true;
                                    appointments[i].Room += (", " + a.Room);
                                    break;
                                }
                            }
                        }
                        if (!isDuplicate)
                        {
                            appointments.Add(a);
                        }
                    }
                }
                dr.Close();
            }
        }
        SQLConn.Close();

        return (appointments.Count > 0) ? appointments : null;
    }

}

