﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Data.SqlClient;

/// <summary>
/// This web service is responsible for account management and login authentication functionality.
/// </summary>
[WebService(Namespace = "http://msis.myvnc.com/webservices/")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
// To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
// [System.Web.Script.Services.ScriptService]
public class UserService : System.Web.Services.WebService {

    public UserService () {

        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }

    /// <summary>
    /// The method accepts a username and password and performs login authentication against the database of 
    /// known users. If the user is successfully authenticated, a UserProfile object is returned to the 
    /// requester containing the details about the user.
    /// </summary>
    /// <param name="username">The username</param>
    /// <param name="password">The SHA1-encrypted hash sum of the password</param>
    /// <returns>A UserProfile object for the user</returns>
    [WebMethod]
    public UserProfile Login(string username, string password)
    {
        UserProfile user = new UserProfile();

        if (!SQLConn.Open())
        {
            return null;
        }

        using (SqlCommand cmd = SQLConn.ExecuteCommand("SELECT * FROM Users WHERE username='" + username + "' AND password='" + password + "'"))
        {
            using (SqlDataReader result = cmd.ExecuteReader())
            {
                if (result.Read())
                {
                    // create userprofile
                    user.IsAuthenticated = true;
                    user.Id = Int32.Parse(result["id"].ToString());
                    user.Name = (string)result["name"];
                    user.Username = (string)result["username"];
                    user.Email = (string)result["email"];
                    user.StudyProgram = (string)result["program_code"];
                    user.Semester = Int32.Parse(result["semester"].ToString());
                    // Create course object

                    string[] courseArray = (result["courses"] as string).Split('|');
                    Course[] courses = new Course[courseArray.Length];
                    for (int i = 0; i < courses.Length; i++)
                    {
                        string code = courseArray[i].Split('#')[0];
                        string desc = courseArray[i].Split('#')[1];
                        bool outsideStudyProgram = bool.Parse(courseArray[i].Split('#')[2]);
                        courses[i] = new Course(code, desc);
                        courses[i].OutsideStudyProgram = outsideStudyProgram;
                    }

                    user.Courses = courses;
                }
                result.Close();
            }
        }
        SQLConn.Close();
        return user;
    }

    /// <summary>
    /// The method should be used to modify the profile of an existing user.
    /// The method accepts a UserProfile object and optionally a new password (encrypted using the 
    /// SHA1 algorithm) for the account. If the password should remain unchanged, just pass an empty string.
    /// </summary>
    /// <param name="user">The user object</param>
    /// <param name="newPassword">The SHA1-encrypted hash sum of the new password (leave empty to keep current password)</param>
    /// <returns>'True' if update was successfull, 'False' otherwise</returns>
    [WebMethod]
    public bool UpdateUser(UserProfile user, string newPassword)
    {
        string coursesSerialized = "";
        for (int i = 0; i < user.Courses.Length; i++)
        {
            Course course = user.Courses[i];
            coursesSerialized += (i == 0)
                ? course.Code + "#" + course.Description + "#" + course.OutsideStudyProgram
                : "|" + course.Code + "#" + course.Description + "#" + course.OutsideStudyProgram;
        }

        if (!SQLConn.Open())
        {
            return false;
        }

        SqlCommand cmd;
        if (newPassword.Length > 0)
        {
            cmd = SQLConn.ExecuteCommand("UPDATE Users SET name='" + user.Name + "',email='" + user.Email + "',username='" + user.Username + "',password='" + newPassword + "',program_code='" + user.StudyProgram + "',courses='" + coursesSerialized + "',semester=" + user.Semester + " WHERE id=" + user.Id);
        }
        else
        {
            cmd = SQLConn.ExecuteCommand("UPDATE Users SET name='" + user.Name + "',email='" + user.Email + "',username='" + user.Username + "',program_code='" + user.StudyProgram + "',courses='" + coursesSerialized + "',semester=" + user.Semester + " WHERE id=" + user.Id);
        }

        using(cmd)
        {
            if (cmd.ExecuteNonQuery() > 0)
            {
                return true;
            }
        }
        SQLConn.Close();
        return false;
    }
    
}

