﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MSIS.webservices.announcements;

namespace MSIS
{
    public partial class AnnouncementsForm : Form
    {
        string username;
        AnnouncementService aService;
        Announcement[] announcements;
        int offset = 0;

        public AnnouncementsForm(string username)
        {
            InitializeComponent();
            
            this.username = username;
         
            try
            {
                aService = new webservices.announcements.AnnouncementService();
                aService.Url = AppSettings.WebServiceUrl + "/" + AppSettings.AnnouncementServiceName;
            }
            catch
            {
                Main.ShowConnectionError();
                Close();
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void Announcements_Load(object sender, EventArgs e)
        {
            try
            {
                // retrieve announcements from web service
                announcements = aService.GetAnnouncements(username);
            }
            catch
            {
                MessageBox.Show("Could not retrieve announcements");
                Close();
            }

            // load the announcements
            if (announcements != null)
            {
                panelAnnouncements.Controls.Remove(lblNoAnnouncements);
                for (int i = 0; i < announcements.Length; i++)
                {
                    AddItem(announcements[i]);
                }
            }
            else
            {
                panelAnnouncements.Controls.Add(lblNoAnnouncements);
            }
        }

        private void menuItemReturn_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void AddItem(Announcement a)
        {
            // add headline
            Label headLabel = new Label();
            headLabel.ForeColor = Color.Black;
            if (a.Category == AnnouncementCategory.Deadline && (a.DateAdded.CompareTo(DateTime.Today) < 0))
            {
                headLabel.ForeColor = Color.DarkGray;
            }
            headLabel.Font = new Font("Tahoma", 8f, FontStyle.Bold);
            headLabel.Location = new Point(4, 4 + offset);
            string dateMonth = (a.DateAdded.Month.ToString().Length == 1) ? "0" + a.DateAdded.Month : a.DateAdded.Month.ToString();
            string dateDay = (a.DateAdded.Day.ToString().Length == 1) ? "0" + a.DateAdded.Day : a.DateAdded.Day.ToString();
            headLabel.Text = a.DateAdded.Year + "-" + dateMonth + "-" + dateDay + ": " + a.Title;
            headLabel.Size = new Size(panelAnnouncements.Width - 30, 15);
            headLabel.Size = new Size(panelAnnouncements.Width - 30, headLabel.Height * CalculateLabelHeight(headLabel));
            panelAnnouncements.Controls.Add(headLabel);

            // add body text
            Label bodyLabel = new Label();
            bodyLabel.ForeColor = Color.Black;
            if (a.Category == AnnouncementCategory.Deadline && (a.DateAdded.CompareTo(DateTime.Today) < 0))
            {
                bodyLabel.ForeColor = Color.Gray;
            }
            bodyLabel.Font = new Font("Tahoma", 8f, FontStyle.Regular);
            bodyLabel.Location = new Point(4, headLabel.Location.Y + headLabel.Size.Height);
            bodyLabel.Text = "[" + a.Category.ToString() + "] " + a.Text;
            bodyLabel.Size = new Size(panelAnnouncements.Width - 30, 15);
            bodyLabel.Size = new Size(panelAnnouncements.Width - 30, bodyLabel.Height * CalculateLabelHeight(bodyLabel));
            panelAnnouncements.Controls.Add(bodyLabel);

            // update offset
            offset = (bodyLabel.Location.Y + bodyLabel.Size.Height) + 10;

        }

        private int CalculateLabelHeight(Label label)
        {
            float dpiFactor = AutoScaleDimensions.Height / 96.0F;
            Graphics g = this.CreateGraphics();
            SizeF size = g.MeasureString(label.Text, label.Font);
            int scaleY = (int)(size.Width / (label.Width - 10)) + 1;
            return Convert.ToInt32(scaleY * dpiFactor);
        }

    }
}

