﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using OpenNETCF.AppSettings;

namespace MSIS
{
    public sealed class AppSettings
    {
        static readonly string AppPath = System.IO.Path.GetDirectoryName(
              System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase);
        static readonly SettingsFile settings = new SettingsFile(AppPath + @"\AppSettings.xml", true);
        static SettingGroup group;

        static AppSettings()
        {
            if (settings.Groups.Count > 0)
            {
                group = settings.Groups["Configuration"];
            }
        }

        AppSettings() { }

        public static void Create()
        {
            if (!ConfigFileExists)
            {
                settings.Groups.Add("Configuration");
                settings.Groups["Configuration"].Settings.Add("WebServiceUrl", "http://localhost/webservices/");
                settings.Groups["Configuration"].Settings.Add("MapUrl", "http://localhost/maps/");
                settings.Groups["Configuration"].Settings.Add("ShowHints", true);
                settings.Save();
            }
            group = settings.Groups["Configuration"];
        }

        public static bool ConfigFileExists
        {
            get
            {
                if (group != null)
                {
                    return true;
                }
                return false;
            }
        }

        public static string WebServiceUrl
        {
            get { return group.Settings["WebServiceUrl"].Value.ToString(); }
            set
            {
                group.Settings["WebServiceUrl"].Value = value;
                settings.Save();
            }
        }

        public static string MapUrl
        {
            get { return group.Settings["MapUrl"].Value.ToString(); }
            set
            {
                group.Settings["MapUrl"].Value = value;
                settings.Save();
            }
        }

        public static bool ShowHints
        {
            get { return (bool)group.Settings["ShowHints"].Value; }
            set
            {
                group.Settings["ShowHints"].Value = value;
                settings.Save();
            }
        }

        public static string UserServiceName
        {
            get { return "UserService.asmx"; }
        }
        public static string LocationServiceName
        {
            get { return "LocationService.asmx"; }
        }
        public static string ScheduleServiceName
        {
            get { return "ScheduleService.asmx"; }
        }
        public static string AnnouncementServiceName
        {
            get { return "AnnouncementService.asmx"; }
        }

        public static string AppTitle
        {
            get { return "MSIS v1.0"; }
        }
    }
}