﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MSIS.webservices.user;

namespace MSIS
{
    public partial class Login : Form
    {
        private UserProfile user = null;
        private bool userIsAuthenticated = false;

        public Login()
        {
            InitializeComponent();
            Text = AppSettings.AppTitle;

            if (!AppSettings.ConfigFileExists)
            {
                ShowConfiguration();
            }

            WiFi wifi = new WiFi();

            if (wifi.Interface == null)
            {
                MessageBox.Show("No supported wireless network interfaces found. This application will not run without a working Wifi connection.",
                                "NIC not found",
                                MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                Application.Exit();
            }
        }

        private void btnLogin_Click(object sender, EventArgs e)
        {
            if (txtUsername.Text.Length > 0 && txtPassword.Text.Length > 0)
            {
                string username = txtUsername.Text;
                // SHA1 hash
                System.Security.Cryptography.SHA1 sha = new System.Security.Cryptography.SHA1CryptoServiceProvider();
                Encoding enc = Encoding.Default;
                string pass_hash = BitConverter.ToString(sha.ComputeHash(enc.GetBytes(txtPassword.Text))).Replace("-", "");

                // Call Login webservice
                UserService myWebService = new UserService();
                try
                {
                    Cursor.Current = Cursors.WaitCursor;
                    user = myWebService.Login(username, pass_hash);
                    Cursor.Current = Cursors.Default;
                    if (user == null)
                    {
                        MessageBox.Show("An error occured: unable to connect to database server.", "Database error",
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                    }
                    else if (user.IsAuthenticated)
                    {
                        // User authenticated OK
                        userIsAuthenticated = true;
                    }
                    else
                    {
                        MessageBox.Show("Wrong username or password!", "Incorrect login",
                            MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Unable to log in. Server not available!", "Error",
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                }
                finally
                {
                    Cursor.Current = Cursors.Default;
                    if (userIsAuthenticated)
                    {
                        // show main menu form
                        new Main(user).ShowDialog();
                    }
                }

            }
        }

        private void Login_Activated(object sender, EventArgs e)
        {
            txtUsername.Text = "";
            txtPassword.Text = "";
        }

        private void menuItemExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void txtUsername_GotFocus(object sender, EventArgs e)
        {
            // enable SIP
            inputPanel.Enabled = true;
        }

        private void txtPassword_GotFocus(object sender, EventArgs e)
        {
            // enable SIP
            inputPanel.Enabled = true;
        }

        private void menuItemShowSettings_Click(object sender, EventArgs e)
        {
            ShowConfiguration();
        }

        private void Login_Load(object sender, EventArgs e)
        {
            panelConfigurationReminder.Visible = !AppSettings.ConfigFileExists;
        }

        private void linkShowConfig_Click(object sender, EventArgs e)
        {
            ShowConfiguration();
        }

        private void ShowConfiguration()
        {
            Configuration config = new Configuration();
            config.ShowDialog();
        }
    }
}