﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using OpenNETCF.Net.NetworkInformation;
using MSIS.webservices.user;

namespace MSIS
{
    public partial class Main : Form
    {
        UserProfile user;
        static WiFi wifiHelper = new WiFi();
        Image tmpImage;

        public Main(UserProfile user)
        {
            InitializeComponent();
            Text = AppSettings.AppTitle;

            this.user = user;
            lblWelcome.Text = "Hi " + user.Name + "!";
        }

        private void menuItemExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void menuItemLogout_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btnAnnouncements_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Sorry, this feature is not yet available...","Not available", 
                MessageBoxButtons.OK,MessageBoxIcon.Asterisk,MessageBoxDefaultButton.Button1);
            // Loads announcements screen
            /*
            Cursor.Current = Cursors.WaitCursor;
            new AnnouncementsForm(user.Username).ShowDialog();
            */
        }

        private void btnSchedule_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            new Schedule(user.Username).ShowDialog();
        }

        private void btnLocationFinder_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            new LocationFinder().ShowDialog();
        }

        public static void ShowConnectionError()
        {
            MessageBox.Show("Unable to connect to server. Please check network status and try again.",
                "No connection", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
        }

        public static void UpdateWifiIndicator(RadioButton indicator)
        {
            if (wifiHelper.IsConnected)
            {
                indicator.Checked = true;
                indicator.Enabled = true;

                string signalStrength = "";
                try
                {
                    signalStrength = " (" + wifiHelper.Interface.SignalStrength.ToString() + ")";
                }
                catch (Exception e) { }

                indicator.Text = "Connected to '" + wifiHelper.Interface.AssociatedAccessPoint + "'" + signalStrength;
            }
            else
            {
                indicator.Checked = false;
                indicator.Enabled = false;
                indicator.Text = "No WiFi Connection!";
            }
        }

        private void mMenuItem_Hover(object sender, MouseEventArgs e)
        {
            switch ((sender as PictureBox).Name)
            {
                case "mMenuAnnouncements":
                    lblMenuDescription.Text = "Important news and notications about your current studies at NTNU.";
                    tmpImage = mMenuAnnouncements.Image;
                    mMenuAnnouncements.Image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                        "MSIS.resources.announcements-hover.png"));
                    break;
                case "mMenuSearch":
                    lblMenuDescription.Text = "Allows you to search for rooms and buildings on the NTNU campus.";
                    tmpImage = mMenuSearch.Image;
                    mMenuSearch.Image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                        "MSIS.resources.location-search-hover.png"));
                    break;
                case "mMenuSchedule":
                    lblMenuDescription.Text = "A dynamically generated schedule of your weekly lectures and exercises.";
                    tmpImage = mMenuSchedule.Image;
                    mMenuSchedule.Image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                        "MSIS.resources.schedule-hover.png"));
                    break;
            }
        }

        private void mMenuItem_MouseUp(object sender, MouseEventArgs e)
        {
            lblMenuDescription.Text = "Please click on one of the items in the menu below to start.";
            (sender as PictureBox).Image = tmpImage;
        }

        /**
        * Timer event handler
        * Continously polls for WiFi reception, and updates the frontend status.
        * 
        */
        private void timer_Tick(object sender, EventArgs e)
        {
            UpdateWifiIndicator(radioWifiStatus);
        }

        private void menuItemSettings_Click(object sender, EventArgs e)
        {
            Configuration config = new Configuration();
            config.ShowDialog();
        }

        private void menuItemAddLocation_Click(object sender, EventArgs e)
        {
            LocationManager lm = new LocationManager();
            lm.ShowDialog();
        }

        public static IEnumerable<Enum> GetEnumValues(Enum enumeration)
        {
            List<Enum> enumerations = new List<Enum>();
            foreach (FieldInfo fieldInfo in enumeration.GetType().GetFields(BindingFlags.Static | BindingFlags.Public))
            {
                enumerations.Add((Enum)fieldInfo.GetValue(enumeration));
            }
            return enumerations;
        }
    }
}