﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using OpenNETCF.Net.NetworkInformation;
using MSIS.webservices.location;

namespace MSIS
{
    public partial class MapWindow : Form
    {

        LocationService locationService;
        MapSheet map;
        Location[] locations = null;

        public MapWindow(string locationName, bool showDescriptionOverlay)
        {
            InitializeComponent();
            Text = AppSettings.AppTitle;
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                locationService = new webservices.location.LocationService();
                locationService.Url = AppSettings.WebServiceUrl + "/" + AppSettings.LocationServiceName;
                locations = locationService.GetLocationByName(locationName);
            }
            catch (Exception e)
            {
                Main.ShowConnectionError();
                Close();
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }

            map = new MapSheet(null);
            mapPanel.Controls.Add(map);
            map.Dock = DockStyle.Top;

            if (locations != null)
            {
                Location location = locations[0];

                map.ClearLocations();

                if (!map.AddLocation(location) || !map.SetSelectedLocationIndex(map.NumberOfLocations - 1))
                {
                    map.ClearLocations();
                    MessageBox.Show("Unable to load map.", "Error",
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                    Close();
                }

                if (showDescriptionOverlay)
                {
                    // update text on map description overlay
                    lblMapDescriptionName.Text = location.Name + ((location.Floor > 0) ? " (floor: " + location.Floor + ")" : "");
                    lblMapDescriptionDesc.Text = location.Description;

                    // enable map description overlay
                    ToggleMapDescription(showDescriptionOverlay);
                }

                // Show map window
                ShowDialog();
            }
            else
            {
                MessageBox.Show("Unable to find a map for this location!", "Oops..",
                    MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                Close();
            }
        }

        private void ToggleMapDescription(bool on)
        {
            if (on)
            {
                map.Size = new Size(Width, mapPanel.Bounds.Height - panelMapOverlay.Bounds.Height);
                lblMapDescriptionDesc.Size = new Size(lblMapDescriptionDesc.Size.Width, CalculateLabelHeight(lblMapDescriptionDesc));

                if (!mapPanel.Controls.Contains(panelMapOverlay))
                {
                    mapPanel.Controls.Add(panelMapOverlay);
                }
                
            }
            else
            {
                map.Size = new Size(Width, Height);
                if (mapPanel.Controls.Contains(panelMapOverlay))
                {
                    mapPanel.Controls.Remove(panelMapOverlay);
                }
            }
        }

        private void menuItemReturn_Click(object sender, EventArgs e)
        {
            Close();
        }

        private int CalculateLabelHeight(Label label)
        {
            float dpiFactor = AutoScaleDimensions.Height / 96.0F;
            Graphics g = this.CreateGraphics();
            SizeF size = g.MeasureString(label.Text, label.Font);
            int scaleY = (int)(size.Width / (label.Width - 5)) + 1;
            return Convert.ToInt32(scaleY * dpiFactor * 14);
        }

    }
}

